@@ -49,7 +49,7 @@ def load_matfile(filepath, variable_names=None, **kwargs):
     variable_names : list of strings
         Top-level variables to be loaded.
     **kwargs : keywords
-        Addidional keyword arguments are passed to :func:`hdf5storage.loadmat`.
+        Additional keyword arguments are passed to :func:`hdf5storage.loadmat`.
 
     Returns
     -------
@@ -209,7 +209,7 @@ def save_RC_h5file(filepath, read_from=None):
         warnings.warn(f"Can't save new RC index. Raised exception: '{err}'.")
 
 
-def load_shcfile(filepath, leap_year=None):
+def load_shcfile(filepath, leap_year=None, comment=None):
     """
     Load shc-file and return coefficient arrays.
 
@@ -220,6 +220,9 @@ def load_shcfile(filepath, leap_year=None):
     leap_year : {True, False}, optional
         Take leap year in time conversion into account (default). Otherwise,
         use conversion factor of 365.25 days per year.
+    comment : str, optional
+        Character at the start of a line to indicate a comment
+        (defaults to ``#``). This can also be a tuple of characters.
 
     Returns
     -------
@@ -238,14 +241,16 @@ def load_shcfile(filepath, leap_year=None):
         piecewise polynomial with ``breaks = time[::step]``.
 
     """
+
     leap_year = True if leap_year is None else leap_year
+    comment = '#' if comment is None else comment
 
     with open(filepath, 'r') as f:
 
         data = np.array([])
         for line in f.readlines():
 
-            if line.strip().startswith('#'):
+            if line.strip().startswith(comment):
                 continue
 
             read_line = np.fromstring(line, sep=' ')
@@ -366,7 +371,7 @@ def save_shcfile(time, coeffs, order=None, filepath=None, nmin=None, nmax=None,
 
 def mjd2000(year, month=1, day=1, hour=0, minute=0, second=0, microsecond=0):
     """
-    Computes the modified Julian date as floating point number.
+    Computes the modified Julian date as floating point number (epoch 2000).
 
     It assigns 0 to 0h00 January 1, 2000. Leap seconds are not accounted for.
 
@@ -431,10 +436,11 @@ datetime.datetime(2002, 3, 4)])
 
         datetime = datetime.astype('datetime64[us]')
 
-        datetime += np.asarray(hour, dtype='timedelta64[h]')
-        datetime += np.asarray(minute, dtype='timedelta64[m]')
-        datetime += np.asarray(second, dtype='timedelta64[s]')
-        datetime += np.asarray(microsecond, dtype='timedelta64[us]')
+        # not use inplace add here because it doesn't broadcast arrays
+        datetime = datetime + np.asarray(hour, dtype='timedelta64[h]')
+        datetime = datetime + np.asarray(minute, dtype='timedelta64[m]')
+        datetime = datetime + np.asarray(second, dtype='timedelta64[s]')
+        datetime = datetime + np.asarray(microsecond, dtype='timedelta64[us]')
 
     microseconds = datetime - np.datetime64('2000-01-01', 'us')
 
@@ -479,7 +485,7 @@ def is_leap_year(year):
 
     Parameters
     ----------
-    year : ndarray, shape (...)
+    year : int, ndarray, shape (...)
         Years to test for leap year.
 
     Returns
@@ -492,8 +498,20 @@ def is_leap_year(year):
     >>> is_leap_year([2000, 2001, 2004])
         array([ True, False,  True])
 
+    Raises
+    ------
+    TypeError if ``year`` is not of type integer.
+
     """
 
+    year = np.asarray(year)
+
+    if not np.issubdtype(year.dtype, int):
+        raise TypeError('Expected integer values as the input year. Use '
+                        'numpy.floor to extract the integer year '
+                        'from decimal years.')
+
+
     return np.logical_and(np.remainder(year, 4) == 0,
                           np.logical_or(np.remainder(year, 100) != 0,
                                         np.remainder(year, 400) == 0))
@@ -532,10 +550,10 @@ def dyear_to_mjd(time, leap_year=None):
     leap_year = True if leap_year is None else leap_year
 
     if leap_year:
-        year = np.asarray(time, dtype=int)
+        year = np.asarray(np.floor(time), dtype=int)  # note: -0.1 is year -1
         frac_of_year = np.remainder(time, 1.)
 
-        isleap = is_leap_year(year)
+        isleap = is_leap_year(year)  # do provide integer years
         days_per_year = np.where(isleap, 366., 365.)
 
         days = frac_of_year * days_per_year
@@ -578,6 +596,9 @@ def mjd_to_dyear(time, leap_year=None):
     >>> mjd_to_dyear([183. , 548.5])  # account for leap years
         array([2000.5, 2001.5])
 
+    >>> mjd_to_dyear([0. , -1., 365.])
+        array([2000., 1999.99726027, 2000.99726776])
+
     >>> mjd_to_dyear([182.625, 547.875], leap_year=False)
         array([2000.5, 2001.5])
 
@@ -586,7 +607,7 @@ def mjd_to_dyear(time, leap_year=None):
     leap_year = True if leap_year is None else leap_year
 
     if leap_year:
-        date = (np.asarray(time, dtype=int)*np.timedelta64(1, 'D')
+        date = (np.asarray(np.floor(time), dtype=int)*np.timedelta64(1, 'D')
                 + np.datetime64('2000-01-01'))  # only precise to date
 
         year = date.astype('datetime64[Y]').astype(int) + 1970
@@ -644,10 +665,10 @@ def gauss_units(deriv=None):
         'nT'
 
     >>> gauss_units(1)
-        '$\\\\mathrm{nT}\\\\cdot\\\\mathrm{yr}^{-1}$'
+        '$\\\\mathrm{nT}\\\\mathrm{yr}^{-1}$'
 
     >>> gauss_units(2)
-        '$\\\\mathrm{nT}\\\\cdot\\\\mathrm{yr}^{-2}$'
+        '$\\\\mathrm{nT}\\\\mathrm{yr}^{-2}$'
 
     """
 
@@ -656,6 +677,6 @@ def gauss_units(deriv=None):
     if deriv == 0:
         units = 'nT'
     else:
-        units = '$\\mathrm{{nT}}\\cdot\\mathrm{{yr}}^{{{:}}}$'.format(-deriv)
+        units = '$\\mathrm{{nT}}\\mathrm{{yr}}^{{{:}}}$'.format(-deriv)
 
     return units
