@@ -20,8 +20,6 @@ field models in general.
 
 import numpy as np
 import warnings
-from numpy import radians, degrees
-from math import pi
 from chaosmagpy.config_utils import basicConfig
 from scipy.interpolate import BSpline, PPoly
 
@@ -81,7 +79,7 @@ def design_matrix(knots, order, n_tdep, time, radius, theta, phi,
 
     assert time.shape == radius.shape == theta.shape == phi.shape
     assert time.ndim == 1
-    assert np.amin(theta) > 0 and np.amax(theta) < degrees(pi)
+    assert np.amin(theta) > 0. and np.amax(theta) < 180.
 
     n_data = time.size  # number of data points
 
@@ -311,7 +309,7 @@ def synth_from_pp(breaks, order, coeffs, time, radius, theta, phi, *,
     if nmax is None:
         nmax = nmax_coeffs
     elif nmax > nmax_coeffs:
-        warnings.warn('Supplied nmax = {0} is incompatible with number of '
+        warnings.warn('Provided nmax = {0} is incompatible with number of '
                       'model coefficients. Using nmax = {1} instead.'.format(
                         nmax, nmax_coeffs))
         nmax = nmax_coeffs
@@ -382,6 +380,36 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
     B_radius, B_theta, B_phi : ndarray, shape (...)
         Radial, colatitude and azimuthal field components.
 
+    Warnings
+    --------
+    The function can also evaluate the field components at the geographic
+    poles, i.e. where ``theta == 0.`` or ``theta == 180.``. However, users
+    should be careful when doing this because the vector basis for spherical
+    geocentric coordinates,
+    :math:`{{\\mathbf{e}_r, \\mathbf{e}_\\theta, \\mathbf{e}_\\phi}}`,
+    depends on longitude, which is not well defined at the poles. That is,
+    at the poles, any value for the longitude maps to the same location in
+    euclidean coordinates but gives a different vector basis in spherical
+    geocentric coordinates. Nonetheless, by choosing a specific value for the
+    longitude at the poles, users can define the vector basis, which then
+    establishes the meaning of the spherical geocentric components returned by
+    this function. The vector basis for the horizontal components is defined as
+
+    .. math::
+
+        \\mathbf{e}_\\theta &= \\cos\\theta\\cos\\phi\\mathbf{e}_x -
+            \\cos\\theta\\sin\\phi\\mathbf{e}_y - \\sin\\theta\\mathbf{e}_z\\\\
+        \\mathbf{e}_\\phi &= -\\sin\\phi\\mathbf{e}_x +
+            \\cos\\phi\\mathbf{e}_y
+
+    Hence, at the geographic north pole as given by ``theta = 0.`` and
+    ``phi = 0.`` (chosen by the user), the returned component ``B_theta``
+    will be along the direction :math:`\\mathbf{e}_\\theta = \\mathbf{e}_x` and
+    ``B_phi`` along :math:`\\mathbf{e}_\\phi = \\mathbf{e}_y`. However,
+    if ``phi = 180.`` is chosen, ``B_theta``
+    will be along :math:`\\mathbf{e}_\\theta = -\\mathbf{e}_x` and ``B_phi``
+    along :math:`\\mathbf{e}_\\phi = -\\mathbf{e}_y`.
+
     Notes
     -----
     The function can work with different grid shapes, but the inputs have to
@@ -414,7 +442,7 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
       theta = np.linspace(0., 180., num=N)  # colatitude of 50 points in deg.
 
       B = cpm.synth_values(coeffs, radius, theta, phi)
-      print([B[num].shape for num in range(3)])  # (N,) shaped output
+      print([B[num].shape for num in range(3)])  # output shape (N,)
 
     Instead of `N` points, compute the field on a regular
     :math:`N\\times N`-grid in azimuth and colatitude (slow).
@@ -422,29 +450,31 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
     .. code-block:: python
 
       radius_grid = 6371.2 * np.ones((N, N))
-      phi_grid, theta_grid = np.meshgrid(phi, theta)  # regular NxN grid
+      phi_grid, theta_grid = np.meshgrid(phi, theta)  # grid of shape (N, N)
 
       B = cpm.synth_values(coeffs, radius_grid, theta_grid, phi_grid)
-      print([B[num].shape for num in range(3)])  # NxN output
+      print([B[num].shape for num in range(3)])  # output shape (N, N)
 
-    But this is slow since some computations on the grid are executed several
-    times. The preferred method is to use NumPy's broadcasting rules (fast).
+    But this is slow since some computations on the grid are repeatedly done.
+    The preferred method is to use NumPy's broadcasting rules (fast).
 
     .. code-block:: python
 
       radius_grid = 6371.2  # float, () or (1,)-shaped array broadcasted to NxN
-      phi_grid = phi[None, ...]  # prepend singleton: 1xN
-      theta_grid = theta[..., None]  # append singleton: Nx1
+      theta_grid = theta[:, None]  # append singleton: (N, 1)
+      phi_grid = phi[None, :]  # prepend singleton: (1, N)
 
       B = cpm.synth_values(coeffs, radius_grid, theta_grid, phi_grid)
-      print([B[num].shape for num in range(3)])  # NxN output
+      print([B[num].shape for num in range(3)])  # output shape (N, N)
 
-    For convenience, you can do the same by using ``grid=True`` option.
+    For convenience, you can do the same by using ``grid=True`` option, which
+    appends a singleton dimension to ``theta`` and inserts a singleton
+    dimension at the second to last position in the shape of ``phi``.
 
     .. code-block:: python
 
       B = cpm.synth_values(coeffs, radius_grid, theta, phi, grid=True)
-      print([B[num].shape for num in range(3)])  # NxN output
+      print([B[num].shape for num in range(3)])  # output shape (N, N)
 
     Remember that ``grid=False`` (default) will result in
     (N,)-shaped outputs as in the first example.
@@ -462,9 +492,12 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
 
     if (theta_min <= 0.0) or (theta_max >= 180.0):
         if (theta_min == 0.0) or (theta_max == 180.0):
-            warnings.warn('The poles are included.')
+            warnings.warn('Provided coordinates include the poles.')
+            poles = True
         else:
             raise ValueError('Colatitude outside bounds [0, 180].')
+    else:
+        poles = False
 
     nmin = 1 if nmin is None else int(nmin)
     assert nmin > 0, 'The value of "nmin" must be greater than zero.'
@@ -499,13 +532,14 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
 
     # manually broadcast input grid on surface
     if grid:
-        theta = theta[..., None]  # first dimension is theta
-        phi = phi[None, ...]  # second dimension is phi
+        theta = np.expand_dims(theta, axis=-1)  # append singleton dimension
+        phi = np.expand_dims(phi, axis=-2)  # insert singleton before last
 
     # get shape of broadcasted result
     try:
-        b = np.broadcast(radius, theta, phi,
-                         np.broadcast_to(0, coeffs.shape[:-1]))
+        b = np.broadcast(
+            radius, theta, phi, np.broadcast_to(0, coeffs.shape[:-1]))
+
     except ValueError:
         print('Cannot broadcast grid shapes (excl. last dimension of coeffs):')
         print(f'radius: {radius.shape}')
@@ -530,8 +564,12 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
     # save sinth for fast access
     sinth = Pnm[1, 1]
 
+    # find the poles
+    if poles:
+        where_poles = (theta == 0.) | (theta == 180.)
+
     # convert phi to radians
-    phi = radians(phi)
+    phi = np.radians(phi)
 
     # allocate arrays in memory
     B_radius = np.zeros(grid_shape)
@@ -540,6 +578,7 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
 
     num = 0
     for n in range(nmin, nmax+1):
+
         if source == 'internal':
             B_radius += (n+1) * Pnm[n, 0] * r_n * coeffs[..., num]
         else:
@@ -548,7 +587,6 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
         B_theta += -Pnm[0, n+1] * r_n * coeffs[..., num]
 
         num += 1
-
         for m in range(1, min(n, mmax)+1):
 
             cmp = np.cos(m*phi)
@@ -558,7 +596,7 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
                 B_radius += ((n+1) * Pnm[n, m] * r_n
                              * (coeffs[..., num] * cmp
                                 + coeffs[..., num+1] * smp))
-            elif source == 'external':
+            else:
                 B_radius += -(n * Pnm[n, m] * r_n
                               * (coeffs[..., num] * cmp
                                  + coeffs[..., num+1] * smp))
@@ -567,8 +605,20 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
                         * (coeffs[..., num] * cmp
                            + coeffs[..., num+1] * smp))
 
-            # divide by sinth later
-            B_phi += (m * Pnm[n, m] * r_n
+            # need special treatment at the poles because Pnm/sinth = 0/0 for
+            # theta in {0., 180.},
+            # use L'Hopital's rule to take the limit:
+            # lim Pnm/sinth = k*dPnm | theta in {0., 180}
+            # (evaluated at poles, where k=1 for theta=0 and k=-1 for
+            # theta=180.); k = costh = P[1, 0] at poles for convenience
+            if poles:
+                div_Pnm = np.where(where_poles, Pnm[m, n+1], Pnm[n, m])
+                div_Pnm[where_poles] *= Pnm[1, 0][where_poles]
+                div_Pnm[~where_poles] /= sinth[~where_poles]
+            else:
+                div_Pnm = Pnm[n, m] / sinth
+
+            B_phi += (m * div_Pnm * r_n
                       * (coeffs[..., num] * smp
                          - coeffs[..., num+1] * cmp))
 
@@ -576,13 +626,9 @@ def synth_values(coeffs, radius, theta, phi, *, nmax=None, nmin=None,
 
         if source == 'internal':
             r_n = r_n / radius  # equivalent to r_n = radius**(-(n+2))
-        elif source == 'external':
+        else:
             r_n = r_n * radius  # equivalent to r_n = radius**(n-1)
 
-    # divide by sinth here
-    sinth = np.where((theta == 0.) | (theta == 180.), np.nan, sinth)
-    B_phi /= sinth
-
     return B_radius, B_theta, B_phi
 
 
@@ -596,12 +642,12 @@ def design_gauss(radius, theta, phi, nmax, *, nmin=None, mmax=None,
     Parameters
     ----------
 
-    radius : ndarray, shape (N,)
-        Array containing the radius of `N` data points in kilometers.
-    theta : ndarray, shape (N,)
+    radius : ndarray, shape (...)
+        Array containing the radius in kilometers.
+    theta : ndarray, shape (...)
         Array containing the colatitude in degrees
         :math:`[0^\\circ,180^\\circ]`.
-    phi : ndarray, shape (N,)
+    phi : ndarray, shape (...)
         Array containing the longitude in degrees.
     nmax : int, positive
         Maximum degree of the sphercial harmonic expansion.
@@ -616,11 +662,42 @@ def design_gauss(radius, theta, phi, nmax, *, nmin=None, mmax=None,
 
     Returns
     -------
-    A_radius, A_theta, A_phi : ndarray, shape (N, M)
+    A_radius, A_theta, A_phi : ndarray, shape (..., M)
         Matrices for radial, colatitude and azimuthal field components. The
         second dimension ``M`` varies depending on ``nmax``, ``nmin`` and
         ``mmax``.
 
+    Warnings
+    --------
+    The function can also return the design matrix for the field components at
+    the geographic poles, i.e. where ``theta == 0.`` or ``theta == 180.``.
+    However, users should be careful when doing this because the vector basis
+    for spherical geocentric coordinates,
+    :math:`{{\\mathbf{e}_r, \\mathbf{e}_\\theta, \\mathbf{e}_\\phi}}`,
+    depends on longitude, which is not well defined at the poles. That is,
+    at the poles, any value for the longitude maps to the same location in
+    euclidean coordinates but gives a different vector basis in spherical
+    geocentric coordinates. Nonetheless, by choosing a specific value for the
+    longitude at the poles, users can define the vector basis, which then
+    establishes the meaning of the spherical geocentric components. The vector
+    basis for the horizontal components is defined as
+
+    .. math::
+
+        \\mathbf{e}_\\theta &= \\cos\\theta\\cos\\phi\\mathbf{e}_x -
+            \\cos\\theta\\sin\\phi\\mathbf{e}_y - \\sin\\theta\\mathbf{e}_z\\\\
+        \\mathbf{e}_\\phi &= -\\sin\\phi\\mathbf{e}_x +
+            \\cos\\phi\\mathbf{e}_y
+
+    Hence, at the geographic north pole as given by ``theta = 0.`` and
+    ``phi = 0.`` (chosen by the user), the returned design matrix ``A_theta``
+    will be for components along the direction
+    :math:`\\mathbf{e}_\\theta = \\mathbf{e}_x` and ``A_phi`` for components
+    along :math:`\\mathbf{e}_\\phi = \\mathbf{e}_y`. However,
+    if ``phi = 180.`` is chosen, ``A_theta`` will be for components along
+    :math:`\\mathbf{e}_\\theta = -\\mathbf{e}_x` and ``A_phi``
+    along :math:`\\mathbf{e}_\\phi = -\\mathbf{e}_y`.
+
     """
 
     # ensure ndarray inputs
@@ -628,9 +705,31 @@ def design_gauss(radius, theta, phi, nmax, *, nmin=None, mmax=None,
     theta = np.asarray(theta, dtype=float)
     phi = np.asarray(phi, dtype=float)
 
-    assert radius.shape == theta.shape == phi.shape
-    assert radius.ndim == 1
-    assert np.amin(theta) >= 0. and np.amax(theta) <= 180.
+    # get shape of broadcasted result
+    try:
+        b = np.broadcast(radius, theta, phi)
+
+    except ValueError:
+        print('Cannot broadcast grid shapes:')
+        print(f'radius: {radius.shape}')
+        print(f'theta:  {theta.shape}')
+        print(f'phi:    {phi.shape}')
+        raise
+
+    shape = b.shape
+
+    theta_min = np.amin(theta)
+    theta_max = np.amax(theta)
+
+    # check if poles are included
+    if (theta_min <= 0.0) or (theta_max >= 180.0):
+        if (theta_min == 0.0) or (theta_max == 180.0):
+            warnings.warn('Provided coordinates include the poles.')
+            poles = True
+        else:
+            raise ValueError('Colatitude outside bounds [0, 180].')
+    else:
+        poles = False
 
     # set internal source as default
     if source is None:
@@ -658,7 +757,11 @@ def design_gauss(radius, theta, phi, nmax, *, nmin=None, mmax=None,
     Pnm = legendre_poly(nmax, theta)
     sinth = Pnm[1, 1]
 
-    phi = radians(phi)
+    phi = np.radians(phi)
+
+    # find the poles
+    if poles:
+        where_poles = (theta == 0.) | (theta == 180.)
 
     # compute the number of dimensions based on nmax, nmin, mmax
     if mmax >= (nmin-1):
@@ -667,39 +770,51 @@ def design_gauss(radius, theta, phi, nmax, *, nmin=None, mmax=None,
         dim = int((nmax-nmin+1)*(2*mmax+1))
 
     # allocate A_radius, A_theta, A_phi in memeory
-    A_radius = np.zeros((theta.size, dim))
-    A_theta = np.zeros((theta.size, dim))
-    A_phi = np.zeros((theta.size, dim))
+    A_radius = np.zeros(shape + (dim,))
+    A_theta = np.zeros(shape + (dim,))
+    A_phi = np.zeros(shape + (dim,))
 
     num = 0
     for n in range(nmin, nmax+1):
-        if source == 'internal':
-            A_radius[:, num] = (n+1) * Pnm[n, 0] * r_n
-        else:
-            A_radius[:, num] = -n * Pnm[n, 0] * r_n
 
-        A_theta[:, num] = -Pnm[0, n+1] * r_n
+        if source == 'internal':
+            A_radius[..., num] = (n+1) * Pnm[n, 0] * r_n
+        else:
+            A_radius[..., num] = -n * Pnm[n, 0] * r_n
+
+        A_theta[..., num] = -Pnm[0, n+1] * r_n
 
         num += 1
-
         for m in range(1, min(n, mmax)+1):
 
             cmp = np.cos(m*phi)
             smp = np.sin(m*phi)
 
             if source == 'internal':
-                A_radius[:, num] = (n+1) * Pnm[n, m] * r_n * cmp
-                A_radius[:, num+1] = (n+1) * Pnm[n, m] * r_n * smp
+                A_radius[..., num] = (n+1) * Pnm[n, m] * r_n * cmp
+                A_radius[..., num+1] = (n+1) * Pnm[n, m] * r_n * smp
             else:
-                A_radius[:, num] = -n * Pnm[n, m] * r_n * cmp
-                A_radius[:, num+1] = -n * Pnm[n, m] * r_n * smp
+                A_radius[..., num] = -n * Pnm[n, m] * r_n * cmp
+                A_radius[..., num+1] = -n * Pnm[n, m] * r_n * smp
 
-            A_theta[:, num] = -Pnm[m, n+1] * r_n * cmp
-            A_theta[:, num+1] = -Pnm[m, n+1] * r_n * smp
+            A_theta[..., num] = -Pnm[m, n+1] * r_n * cmp
+            A_theta[..., num+1] = -Pnm[m, n+1] * r_n * smp
 
-            # divide by synth after building the matrix
-            A_phi[:, num] = m * Pnm[n, m] * r_n * smp
-            A_phi[:, num+1] = -m * Pnm[n, m] * r_n * cmp
+            # need special treatment at the poles because Pnm/sinth = 0/0 for
+            # theta in {0., 180.},
+            # use L'Hopital's rule to take the limit:
+            # lim Pnm/sinth = k*dPnm | theta in {0., 180}
+            # (evaluated at poles, where k=1 for theta=0 and k=-1 for
+            # theta=180.); k = costh = P[1, 0] at poles for convenience
+            if poles:
+                div_Pnm = np.where(where_poles, Pnm[m, n+1], Pnm[n, m])
+                div_Pnm[where_poles] *= Pnm[1, 0][where_poles]
+                div_Pnm[~where_poles] /= sinth[~where_poles]
+            else:
+                div_Pnm = Pnm[n, m] / sinth
+
+            A_phi[..., num] = m * div_Pnm * r_n * smp
+            A_phi[..., num+1] = -m * div_Pnm * r_n * cmp
 
             num += 2
 
@@ -709,10 +824,6 @@ def design_gauss(radius, theta, phi, nmax, *, nmin=None, mmax=None,
         else:
             r_n = r_n * radius
 
-    # divide phi component by sinth
-    sinth = np.where((theta == 0.) | (theta == 180.), np.nan, sinth)
-    A_phi /= sinth[:, None]
-
     return A_radius, A_theta, A_phi
 
 
@@ -746,7 +857,7 @@ def legendre_poly(nmax, theta):
 
     """
 
-    costh = np.cos(radians(theta))
+    costh = np.cos(np.radians(theta))
     sinth = np.sqrt(1-costh**2)
 
     Pnm = np.zeros((nmax+1, nmax+2) + costh.shape)
